
#***********************************
#Goodness of fit for GLLVM for ordinal data
#Cagnone, S., Zhu, J., and Reiser, M. 2022
#*************************************
#**********************************************************************************************
# This code produces goodness-of-fit statistics for the GLLVM IRT model fit to
# cross-classified variables.  The following statistics are produced:
# Pearson-Fisher chi-square statistic
# X^2_[2] on second-order marginals
# Extended Gffit_ij on orthogonal components
# M_ij
# X^2-bar_ij
# X^2-doublebar_ij
# *** The fortran module gfi.f90 was compiled on a linux workstation with a 64 bit processor.
# If you are using a MS Windows system, you will need to recompile gfi.f90 on your local 
# workstation.  Even if you are using linux, you may need to recompile gfi.f90 on
# your local workstation. ***
# # The model specifications are currently set to seven variables each with four
# categories.  The application in the paper useses data on seven depression symptoms
# with four response categories from Afifi, A.A., & Clark, V. (1984). Computer-Aided 
# Multivariate Analysis, Lifetime Learning Publications, Belmont: CA.  The data are
# also reproduced in Sharma, S. (1995). Applied Multivariate Techniques. Wiley, New York,
# and are available from 
# https://bcs.wiley.com/he-bcs/Books?action=chapter&bcsId=4568&itemId=0471310646&chapterId=42945
# The data set is depress.dat and documentation is in the file depress.doc. 
# The symptom variables in the application from the data set are 
# (1) "I felt that I could not shake off the blues even with the help of my family and friends."
# (2) "I felt hopeful about the future."
# (3) "I felt that everything was an effort."
# (4) "I felt lonely." 
# (5) "I felt fearful." 
# (6) "I thought my life had become a failure"
# and (7) "I felt that people disliked me." 
# Using these seven variables from the Afifi & Clark dataset, the code produces the
# following output:
# [1] 1
# [1] "thresholds"
#             [,1]       [,2]       [,3]      [,4]      [,5]      [,6]      [,7]
# beta.1 -4.638887 -3.0953974 -2.6125219 -4.459713 -4.826731 -8.462631 -6.067357
# beta.2 -3.451611 -1.9441573 -1.6387767 -2.736991 -4.113430 -6.370210 -4.228954
# beta.3 -1.764164 -0.7896233 -0.2466003 -1.132911 -2.228114 -4.134587 -2.230979
# [1] "slopes"
#          [,1]     [,2]      [,3]    [,4]     [,5]     [,6]     [,7]
# [1,] 2.023759 1.092122 0.9232988 2.27143 1.817864 4.120272 2.046357 
#
# [1] "Pearson-Fisher Chi-square Statistic"
# [1] 11515.15
# [1] "X^2_[2] Statistic"
# [1] 222.6196
# [1] "Extended GFfit_ij"
#      [,1]     [,2]     [,3]      [,4]     [,5]     [,6]      [,7]
# [1,]    0 25.94826 6.050207  9.627805 10.17947 7.341253  3.860644
# [2,]    0  0.00000 3.414699 12.408345 13.97921 7.105545 14.816244
# [3,]    0  0.00000 0.000000 20.160735 20.39668 9.285370  4.811657
# [4,]    0  0.00000 0.000000  0.000000 13.16612 4.465212 10.539461
# [5,]    0  0.00000 0.000000  0.000000  0.00000 7.190086 11.172717
# [6,]    0  0.00000 0.000000  0.000000  0.00000 0.000000  6.699910
# [1] "M_ij"
#    1        2        3         4         5         6         7
# 1 NA 18.93568 3.881743 13.189772  9.748467 10.233792  6.660662
# 2 NA       NA 2.700114  9.365069 12.775806  9.257751 17.115267
# 3 NA       NA       NA 14.053772 21.204549  6.094345  9.908170
# 4 NA       NA       NA        NA  7.592219  6.886057 12.711511
# 5 NA       NA       NA        NA        NA  5.935851 15.769424
# 6 NA       NA       NA        NA        NA        NA 10.042990
# [1] "Pearson_ij or X^2_ij"
#   1        2        3         4         5         6         7
# 1 NA 18.93568 3.881743 13.189772  9.748467 10.233792  6.660662
# 2 NA       NA 2.700114  9.365069 12.775806  9.257751 17.115267
# 3 NA       NA       NA 14.053772 21.204549  6.094345  9.908170
# 4 NA       NA       NA        NA  7.592219  6.886057 12.711511
# 5 NA       NA       NA        NA        NA  5.935851 15.769424
# 6 NA       NA       NA        NA        NA        NA 10.042990
# [1] "X^2-bar_ij"
#    1        2        3         4         5         6         7
# 1 NA 19.75816 4.048930 13.766015 10.190233 10.719698  6.971155
# 2 NA       NA 2.795486  9.763246 13.299027  9.753419 17.865151
# 3 NA       NA       NA 14.649181 22.052920  6.428550 10.336660
# 4 NA       NA       NA        NA  7.929902  7.202557 13.289247
# 5 NA       NA       NA        NA        NA  6.224474 16.492242
# 6 NA       NA       NA        NA        NA        NA 10.524305
# [1] "X^2-doublebar_ij"
#    1        2        3         4         5        6         7
# 1 NA 16.58341 2.668933 11.434016  8.128920 8.678311  5.282490
# 2 NA       NA 1.544071  7.831447 10.815618 7.645099 14.849539
# 3 NA       NA       NA 12.175896 18.524612 4.680468  8.186016
# 4 NA       NA       NA        NA  6.185576 5.577277 10.957948
# 5 NA       NA       NA        NA        NA 4.652937 13.682613
# 6 NA       NA       NA        NA        NA       NA  8.503033
#           gffit   pvalgffit pvalgffitfdr     xdbar   pvalxdbar pvalxdbarBY pvalxdbarfdr
#  [1,] 25.948256 0.002083194   0.04374707 16.583415 0.020289168   0.7765936    0.2130363
#  [2,]  6.050207 0.734882993   0.90779664  2.668933 0.913848421   1.0000000    0.9595408
#  [3,]  9.627805 0.381447688   0.71988012 11.434016 0.120775976   1.0000000    0.3855242
#  [4,] 10.179473 0.336152284   0.70591980  8.128920 0.321363365   1.0000000    0.5473889
#  [5,]  7.341253 0.601636231   0.87658408  8.678311 0.276584379   1.0000000    0.5473889
#  [6,]  3.860644 0.920342700   0.94556539  5.282490 0.625532222   1.0000000    0.7727163
#  [7,]  3.414699 0.945565394   0.94556539  1.544071 0.980744595   1.0000000    0.9807446
#  [8,] 12.408345 0.191255141   0.57376542  7.831447 0.347691310   1.0000000    0.5473889
#  [9,] 13.979211 0.123066641   0.51687989 10.815618 0.146866351   1.0000000    0.3855242
# [10,]  7.105545 0.626131482   0.87658408  7.645099 0.364925959   1.0000000    0.5473889
# [11,] 14.816244 0.096108679   0.50457056 14.849539 0.037977193   0.9690834    0.2658403
# [12,] 20.160735 0.016945949   0.11862164 12.175896 0.094920810   1.0000000    0.3855242
# [13,] 20.396676 0.015616110   0.11862164 18.524612 0.009814465   0.7513222    0.2061038
# [14,]  9.285370 0.411360067   0.71988012  4.680468 0.698890951   1.0000000    0.7761505
# [15,]  4.811657 0.850407766   0.94556539  8.186016 0.316477467   1.0000000    0.5473889
# [16,] 13.166121 0.155232416   0.54331346  6.185576 0.518255592   1.0000000    0.7255578
# [17,]  4.465212 0.878218937   0.94556539  5.577277 0.589879876   1.0000000    0.7727163
# [18,] 10.539461 0.308600769   0.70591980 10.957948 0.140463839   1.0000000    0.3855242
# [19,]  7.190086 0.617336087   0.87658408  4.652937 0.702231372   1.0000000    0.7761505
# [20,] 11.172717 0.264055646   0.69314607 13.682613 0.057123059   1.0000000    0.2998961
# [21,]  6.699910 0.668330524   0.87718381  8.503033 0.290330419   1.0000000    0.5473889
#
#***********************************************************************************************


library(ltm)
#library(doParallel)
library(matrixcalc)
library(mvtnorm)
library(fastGHQuad)
library(combinat)
library(MASS)
library(Matrix)
#   library(foreach)


source("Estep.R")
source("derivative.R")
source("zex.R")
source("estimate_function.R")

#********* Input information **************
#

# nsamp is always 1 since we are not doing simulation here
adjust=F                #True: put a cap on parameter estimates
adjust_w=0              #add this value to the diagnal of the w matrix to make sure it's not singular
n <-294                #sample size
szo  <-4                #nr of categories per each variable
npo  <-7                #nr of observed variables

nfac<-1                 #nr of latent variables used in the model
nq   <- 21              #nr of quadrature points
ncol1 <- nq^nfac        #grid of quadrature points
nsamp <-1             #number of Monte Carlo replications

error_samp<-NULL

chisqCM<-NULL
chisqJO<-NULL
chisqRe<-NULL
chisqPe<-NULL


GffitMJ<-matrix(0, npo-1,npo)
GffitCM<-matrix(0, npo-1,npo)
GffitRE<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))     #orthogonal GFfit
chisqJO_ij<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))  # M_ij
Pearson_ij<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))  # X_ij
x_d_bar_ij<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))  # X_double_bar
x_bar_ij<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))    # X_bar
mu_1_ij<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))     # mu_1 and mu_2 used to compute x_bar and x_double_bar
mu_2_ij<-array(NA,dim=c(npo-1,npo,nsamp),dimnames=list(1:(npo-1),1:npo,1:nsamp))

iresp_tot<-z_ext(c(1:szo),npo)  #all the response patterns
n_tot<-dim(iresp_tot)[1]

#**********generating M matrices********************************************************


#M first
index<-NULL
cc<-1
i_s<-vector()
j_s<-vector()
for (i in 1:npo) {
  
  for (k in 1:(szo-1)){
    j_temp<-which(iresp_tot[,i]==(szo-k+1))
    j_s<-c(j_s,j_temp)
    i_s<-c(i_s,rep(cc,length(j_temp)))
    cc<-cc+1
  }
}
Mfirst<-sparseMatrix(i=i_s,j=j_s,x=rep(1,length(i_s)))



#****************Msecondfull****************************

indexfull<-NULL
#indexfull indicates the coresponding variable pair for each row of the Msecondfull matrix
ccc=1
i_s<-vector()
j_s<-vector()
for (i in 1:(npo-1)) {
  
  for (j in (i+1):npo){
    for (k in 1:(szo)){
      for (k1 in 1:(szo)) {
        indexfull<-rbind(indexfull,c(i,j))
        j_temp<-which(iresp_tot[,i]==(szo-k+1) & iresp_tot[,j]==szo-k1+1)
        j_s<-c(j_s,j_temp)
        i_s<-c(i_s,rep(ccc,length(j_temp)))
        ccc<-ccc+1
      }
    }
  }
}
Msecondfull<-sparseMatrix(i=i_s,j=j_s,x=rep(1,length(i_s)))


#choosecell will generate all the cells used to compute orthogonal GFfit
# if keep1 is not specified, by default use the right-bottom (c-1)^2 cells for each variable pairs

choosecell<-function(npo,szo,keep1=NULL){
  if(is.null(keep1)){
    for(i in 1:(szo-1)){
      keep1<-c(keep1,(1:(szo-1))+szo*(i-1))
    }
    keep2<-keep1
    for(i in 1:(choose(npo,2)-1)){
      keep2<-c(keep2,keep1+i*szo^2)
    }
  }else{
    keep2<-keep1
    for(i in 1:(choose(npo,2)-1)){
      keep2<-c(keep2,keep1+i*szo^2)
    }
  }
  return(keep2)
}

#you can specify your own keep1 below, you can put cells in the center either earlier or later in the keep1 vector

keep1<-NULL
#keep1=c(1:5,7:11,13,14,17,19,20,23, 25:29,15,16,21,22)

keep<-choosecell(npo=npo,szo=szo,keep1)


index=indexfull[keep,]   #index is for the H matrix, indexfull is for the M matrix in the paper


##*********************read dataset*****************************
# The data set for the application in the paper may be downloaded from
# https://bcs.wiley.com/he-bcs/Books?action=chapter&bcsId=4568&itemId=0471310646&chapterId=42945
mydata<-read.table('c1c9c14c3c6c7c20depress.dat') #change your path here
#mydata<-mydata[,c(2,3,4,5,7)]+1 #if your category starts from 0, +1 makes it start from 1 since the code needs it to be in this form
# reorder categories from lowest to highest if necessary.
# Example three categories
#for(i in 1:47){
#  for(j in 1:5){
#    if(mydata[i,j]==1){
#      mydata[i,j]=3
#    }else if(mydata[i,j]==3){
#      mydata[i,j]=1
#    }
#  }
#}

iresp1<-mydata

omit=TRUE

(result<-estimate_func_grm(1,omit,adjust=adjust,adjust_w = adjust_w))

attach(result)
print("Pearson-Fisher Chi-square Statistic")
print(chisqPea)
print("X^2_[2] Statistic")
print(chisqR)
print("GFfit_ij")
print(result$Gf_fitO)
print("M_ij")
print(result$chisqJO_ij)
print("Pearson_ij or X^2_ij")
print(result$Pearson_ij)
print("X^2-bar_ij")
print(result$x_bar_ij)
print("X^2-doublebar_ij")
print(result$x_d_bar_ij)

x <- result$Gf_fitO
xtran <- t(x)
xlt <- xtran[lower.tri(xtran)]
dfgffit <- (szo-1)*(szo-1)
pvalgffit <- 1-pchisq(xlt,dfgffit)
gffit <- xlt
x <- result$x_d_bar_ij
xtran <- t(x)
plength <- szo*szo/2
pvalxdbar <- rep(NA,plength)
pvalxdbarfdr <- rep(NA,plength)
pvalxdbarBY <- rep(NA,plength)
xlt <- xtran[lower.tri(xtran)]
dfxdbar <- szo*szo-2*szo-1
if(szo > 2) {pvalxdbar <- 1-pchisq(xlt,dfxdbar)}
xdbar <- xlt
pvalgffitfdr <- p.adjust(pvalgffit,"fdr")
if(szo >2) pvalxdbarfdr <- p.adjust(pvalxdbar,"fdr")
if (szo >2) pvalxdbarBY <- p.adjust(pvalxdbar,"BY")
dfpval <- cbind(gffit,pvalgffit,pvalgffitfdr,xdbar,pvalxdbar,pvalxdbarBY,pvalxdbarfdr)
print(dfpval)












